import { Element } from 'cheerio';
import { Context, SkinContext, CapeContext, ServerContext, IContextOptions } from './';
import { kSerializeData } from '../utils';
import { IFriend } from '../api';
export interface IPlayerContextOptions extends IContextOptions {
    data?: string | Element | Element[];
    isSearch?: boolean;
}
export declare type Username = string;
export declare type Hash = string;
export interface IUsername {
    username: Username;
    changed_at: string | null;
    timestamp: number | null;
}
export declare enum Sort {
    ASC = "asc",
    DESC = "desc"
}
export declare type SortUnion = `${Sort}`;
export interface IGetFollowersOptions {
    /**
     * Sort filter
     */
    sort?: FollowersSort;
    /**
     * Page number
     */
    page?: number;
}
export declare type FollowersSort = Partial<Record<'profile' | 'date' | 'following', Sort | SortUnion>>;
/**
 * @hidden
 */
export declare enum FollowersSection {
    FOLLOWING = "following",
    FOLLOWERS = "followers"
}
/**
 * @hidden
 */
export declare type FollowersSectionUnion = `${FollowersSection}`;
/**
 * @hidden
 */
export interface ILoadFollowersOptions extends IGetFollowersOptions {
    section: FollowersSection | FollowersSectionUnion;
}
export declare class PlayerContext extends Context<PlayerContext> {
    /**
     * Profile ID
     */
    readonly id: number;
    /**
     * Player uuid
     */
    readonly uuid: string;
    /**
     * Player username
     */
    readonly username: string;
    /**
     * Profile short url
     */
    readonly url: string;
    /**
     * Profile views
     */
    readonly views: number;
    /**
     * Nickname history
     */
    readonly names: IUsername[];
    /**
     * Skin history
     */
    readonly skins: SkinContext[];
    /**
     * Player capes
     */
    readonly capes: CapeContext[];
    /**
     * Player friends
     */
    readonly friends: IFriend[];
    /**
     * Player favorite servers
     */
    readonly servers: ServerContext[];
    /**
     * Player followers
     */
    readonly followers: PlayerContext[];
    /**
     * Player following
     */
    readonly following: PlayerContext[];
    /**
     * Following date, available when receiving player followers/following
     */
    readonly followingDate: number;
    /**
     * Badlion Client statistics
     *
     * @see {@link https://www.badlion.net/forum/thread/309559 | Announce from Badlion Client}
     */
    readonly badlion: {
        /**
         * Total time spent in the game
         */
        play_time: number;
        /**
         * Login streak
         */
        login_streak: {
            /**
             * Current value
             */
            current: number;
            /**
             * Max value
             */
            max: number;
        };
        /**
         * Last game server
         */
        last_server: string;
        /**
         * Last online
         */
        last_online: number;
        /**
         * Used game version
         */
        version: string;
    } | null;
    /**
     * Payload loaded
     * @hidden
     */
    private extended;
    /**
     * @hidden
     */
    constructor({ data, isSearch, ...options }: IPlayerContextOptions);
    /**
     * Check payload loaded
     */
    get isExtended(): boolean;
    /**
     * Does the player use Badlion Client
     */
    get usesBadlion(): boolean;
    /**
     * Get clean UUID
     */
    get cleanUUID(): string;
    /**
     * Load all profile information
     */
    loadPayload(): Promise<void>;
    /**
     * Get player followers
     */
    getFollowers(options: IGetFollowersOptions): Promise<PlayerContext['followers']>;
    /**
     * Get player following
     */
    getFollowing(options: IGetFollowersOptions): Promise<PlayerContext['followers']>;
    /**
     * @hidden
     */
    private loadFollowers;
    /**
     * @hidden
     */
    private parseFollowers;
    /**
     * @hidden
     */
    private parseUsernameHistory;
    /**
     * @hidden
     */
    [kSerializeData](): any;
}
