import * as cheerio from 'cheerio';
import { Context, IContextOptions } from './context';
import { RendersContext } from './renders';
import { Hash } from './player';
import { kSerializeData } from '../utils';
export interface ISkinContextOptions extends IContextOptions<SkinContext> {
    data?: string | cheerio.Element | cheerio.Element[];
    type?: 'extended';
}
/**
 * Skin model type
 */
export declare enum Model {
    UNKNOWN = "unknown",
    CLASSIC = "classic",
    SLIM = "slim"
}
export declare type ModelUnion = `${Model}`;
/**
 * Skin transformation type
 */
export declare enum Transformation {
    GRAYSCALE = "grayscale",
    INVERT = "invert",
    ROTATE_HUE_180 = "rotate-hue-180",
    ROTATE_HEAD_LEFT = "rotate-head-left",
    ROTATE_HEAD_RIGHT = "rotate-head-right",
    HAT_PUMPKIN_MASK_1 = "hat-pumpkin-mask-1",
    HAT_PUMPKIN_MASK_2 = "hat-pumpkin-mask-2",
    HAT_PUMPKIN_MASK_3 = "hat-pumpkin-mask-3",
    HAT_PUMPKIN_MASK_4 = "hat-pumpkin-mask-4",
    HAT_PUMPKIN = "hat-pumpkin",
    HAT_PUMPKIN_CREEPER = "hat-pumpkin-creeper",
    HAT_SANTA = "hat-santa"
}
export declare type TransformationUnion = `${Transformation}`;
export interface ITransformSkinOptions {
    /**
     * Skin id
     */
    skin: Hash;
    /**
     * Skin transformation type
     */
    transformation?: Transformation | TransformationUnion;
    /**
     * Skin model
     */
    model?: Model | ModelUnion;
}
export declare class SkinContext extends Context<SkinContext> {
    /**
     * Skin id
     */
    readonly id: string;
    /**
     * Skin name
     */
    readonly name: string;
    /**
     * Skin model
     */
    readonly model: Model | ModelUnion;
    /**
     * Skin tags
     */
    readonly tags: string[];
    /**
     * Skin rating
     */
    readonly rating: number;
    /**
     * Skin creation timestamp
     */
    readonly createdAt: number;
    /**
     * Skin transformation type
     */
    readonly transformation: Transformation | TransformationUnion | null;
    /**
     * Payload loaded
     * @hidden
     */
    private extended;
    /**
     * @hidden
     */
    constructor({ data, type, ...options }: ISkinContextOptions);
    /**
     * Get skin url
     */
    get url(): string;
    /**
     * Get skin renders
     */
    get renders(): RendersContext;
    /**
     * Check is slim skin model
     */
    get isSlim(): boolean;
    /**
     * Check payload loaded
     */
    get isExtended(): boolean;
    /**
     * Check is transformed
     */
    get isTransformed(): boolean;
    /**
     * Transform skin
     *
     * @see {@link https://namemc.com/skin/ee40191789e621d3 | Check "Tools" card}
     */
    transform(transformation: Transformation | TransformationUnion): Promise<SkinContext>;
    /**
     * Load all skin information
     */
    loadPayload(): Promise<void>;
    /**
     * @hidden
     */
    static parseSkinLink(link: string): Pick<SkinContext, 'id' | 'model'> | void;
    /**
     * @hidden
     */
    protected parseSkinRating($: cheerio.CheerioAPI): number;
    /**
     * @hidden
     */
    protected parseSkinTime($: cheerio.CheerioAPI): number;
    /**
     * @hidden
     */
    [kSerializeData](): any;
}
