"use strict";

exports.Transformation = exports.SkinContext = exports.Model = void 0;

var cheerio = require("cheerio");

var _context = require("./context");

var _renders = require("./renders");

var _error = require("../error");

var _utils = require("../utils");

/**
 * Skin model type
 */
var Model;
exports.Model = Model;

(function (Model) {
  Model["UNKNOWN"] = "unknown";
  Model["CLASSIC"] = "classic";
  Model["SLIM"] = "slim";
})(Model || (exports.Model = Model = {}));
/**
 * Skin transformation type
 */


var Transformation;
exports.Transformation = Transformation;

(function (Transformation) {
  Transformation["GRAYSCALE"] = "grayscale";
  Transformation["INVERT"] = "invert";
  Transformation["ROTATE_HUE_180"] = "rotate-hue-180";
  Transformation["ROTATE_HEAD_LEFT"] = "rotate-head-left";
  Transformation["ROTATE_HEAD_RIGHT"] = "rotate-head-right";
  Transformation["HAT_PUMPKIN_MASK_1"] = "hat-pumpkin-mask-1";
  Transformation["HAT_PUMPKIN_MASK_2"] = "hat-pumpkin-mask-2";
  Transformation["HAT_PUMPKIN_MASK_3"] = "hat-pumpkin-mask-3";
  Transformation["HAT_PUMPKIN_MASK_4"] = "hat-pumpkin-mask-4";
  Transformation["HAT_PUMPKIN"] = "hat-pumpkin";
  Transformation["HAT_PUMPKIN_CREEPER"] = "hat-pumpkin-creeper";
  Transformation["HAT_SANTA"] = "hat-santa";
})(Transformation || (exports.Transformation = Transformation = {}));

class SkinContext extends _context.Context {
  /**
   * @hidden
   */
  constructor({
    data,
    type,
    ...options
  }) {
    super(options);
    /**
     * Skin id
     */

    this.id = '';
    /**
     * Skin name
     */

    this.name = '';
    /**
     * Skin model
     */

    this.model = Model.UNKNOWN;
    /**
     * Skin tags
     */

    this.tags = [];
    /**
     * Skin rating
     */

    this.rating = 0;
    /**
     * Skin creation timestamp
     */

    this.createdAt = 0;
    /**
     * Skin transformation type
     */

    this.transformation = null;
    /**
     * Payload loaded
     * @hidden
     */

    this.extended = false;
    this.setupPayload();

    if (!data) {
      return;
    }

    switch (type) {
      case 'extended':
        {
          this.extended = true;
          const $ = cheerio.load(data);
          const {
            attribs: {
              href
            }
          } = $('#render-button.btn').get(0);
          const skin = SkinContext.parseSkinLink(href);

          if (skin) {
            const {
              id,
              model
            } = skin;
            this.id = id;
            this.model = model;
            this.rating = this.parseSkinRating($);
            this.createdAt = this.parseSkinTime($);
            this.tags = $('div.card-body.text-center.py-1 > a.badge.badge-pill').map((index, element) => {
              // @ts-ignore
              const {
                children: [{
                  data: tag
                }]
              } = element;
              return tag;
            }).get();
          }

          break;
        }

      default:
        {
          var _data$parent, _data$parent2, _cardHeader$children$;

          // @ts-ignore
          const cardLinkHash = data === null || data === void 0 ? void 0 : (_data$parent = data.parent) === null || _data$parent === void 0 ? void 0 : _data$parent.attribs.href.replace(_utils.skinRegExp, '$1');
          const cardHeader = data === null || data === void 0 ? void 0 : (_data$parent2 = data.parent) === null || _data$parent2 === void 0 ? void 0 : _data$parent2.children // @ts-ignore
          .filter(({
            name,
            attribs: {
              class: className = ''
            } = {}
          }) => name === 'div' && className.includes('card-header'))[0];
          const cardName = cardHeader ? // @ts-ignore
          (_cardHeader$children$ = cardHeader.children[0]) === null || _cardHeader$children$ === void 0 ? void 0 : _cardHeader$children$.data : '';
          const $ = cheerio.load(data);
          const [{
            id,
            model
          }] = $('div > img.drop-shadow').map((index, {
            attribs: {
              'data-src': src
            }
          }) => {
            const skin = SkinContext.parseSkinLink(src);
            return {
              id: cardLinkHash,
              model: Model.UNKNOWN,
              ...skin
            };
          }).get();
          this.id = id;
          this.model = model;
          this.name = cardName;
          this.rating = this.parseSkinRating($);
        }
    }
  }
  /**
   * Get skin url
   */


  get url() {
    return `${this.options.getEndpoint()}/texture/${this.id}.png`;
  }
  /**
   * Get skin renders
   */


  get renders() {
    const {
      options,
      client,
      api
    } = this;
    return new _renders.RendersContext({
      skin: this.id,
      model: this.model,
      options,
      client,
      api
    });
  }
  /**
   * Check is slim skin model
   */


  get isSlim() {
    return this.model === Model.SLIM;
  }
  /**
   * Check payload loaded
   */


  get isExtended() {
    return this.extended;
  }
  /**
   * Check is transformed
   */


  get isTransformed() {
    return Boolean(this.transformation);
  }
  /**
   * Transform skin
   *
   * @see {@link https://namemc.com/skin/ee40191789e621d3 | Check "Tools" card}
   */


  transform(transformation) {
    return this.client.post('/transform-skin', `skin=${this.id}&transformation=${transformation}`, {
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
        origin: 'https://ru.namemc.com'
      }
    }).then(({
      request
    }) => {
      var _request$res;

      const [, hash] = ((request === null || request === void 0 ? void 0 : (_request$res = request.res) === null || _request$res === void 0 ? void 0 : _request$res.responseUrl) || request.responseURL).match(_utils.skinRegExp);

      if (!hash) {
        throw new _error.WrapperError('NO_USEFUL');
      }

      this.payload = {
        hash,
        transformation
      };
      this.setupPayload();
      return this;
    }).catch(error => {
      var _error$response;

      throw (error === null || error === void 0 ? void 0 : (_error$response = error.response) === null || _error$response === void 0 ? void 0 : _error$response.status) === 404 ? new _error.WrapperError('NOT_FOUND', this.id) : error;
    });
  }
  /**
   * Load all skin information
   */


  async loadPayload() {
    if (this.isExtended) {
      return;
    }

    this.payload = await this.client.get(`/skin/${this.id}`).then(({
      data
    }) => {
      const skin = new SkinContext({ ...this,
        data,
        type: 'extended',
        payload: {
          name: this.name
        }
      });

      if (!skin.id) {
        throw new _error.WrapperError('NO_USEFUL');
      }

      return skin;
    });
    this.setupPayload();
    this.extended = true;
  }
  /**
   * @hidden
   */


  static parseSkinLink(link) {
    const {
      searchParams,
      host
    } = new URL(link);
    const id = searchParams.get('id');

    if (id) {
      _renders.RendersContext.DOMAIN = host;
      return {
        id,
        model: searchParams.get('model') || Model.UNKNOWN
      };
    }
  }
  /**
   * @hidden
   */


  parseSkinRating($) {
    var _$$get;

    const ratingElement = ((_$$get = $('.position-absolute.bottom-0.right-0.text-muted').get(0)) === null || _$$get === void 0 ? void 0 : _$$get.children) || null;
    const rating = ratingElement ? (ratingElement.length > 1 ? ratingElement[1] : ratingElement[0] // @ts-ignore Invalid lib types
    ).data : '0';
    return Number(rating.replace(/[^\d]+([\d]+)/, '$1'));
  }
  /**
   * @hidden
   */


  parseSkinTime($) {
    const date = $('.position-absolute.bottom-0.left-0.text-muted.title-time').get(0).attribs.title;
    return new Date(date).getTime();
  }
  /**
   * @hidden
   */


  [_utils.kSerializeData]() {
    return (0, _utils.pickProperties)(this, ['id', 'name', 'url', 'model', 'isSlim', 'transformation', 'isTransformed', 'renders', 'tags', 'rating', 'createdAt']);
  }

}

exports.SkinContext = SkinContext;